//*****************************************************************************
//
//! \file w6100.c
//! \brief W6100 HAL Implements file.
//! \version 1.0.0
//! \date 2019/01/01
//! \par  Revision history
//!       <2019/01/01> 1st Release
//! \author MidnightCow
//! \copyright
//!
//! Copyright (c)  2019, WIZnet Co., LTD.
//!
//! Permission is hereby granted, free of charge, to any person obtaining a copy
//! of this software and associated documentation files (the "Software"), to deal
//! in the Software without restriction, including without limitation the rights 
//! to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//! copies of the Software, and to permit persons to whom the Software is 
//! furnished to do so, subject to the following conditions: 
//!
//! The above copyright notice and this permission notice shall be included in
//! all copies or substantial portions of the Software. 
//!
//! THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//! IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//! FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//! AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//! LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//! OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
//! SOFTWARE. 
//!
//*****************************************************************************

#include "w6100.h"
#include "SPI.h"
#include <stdio.h>
#include "w6100_init.h"

#define _WIZCHIP_SPI_VDM_OP_    0x00
#define _WIZCHIP_SPI_FDM_LEN1_  0x01
#define _WIZCHIP_SPI_FDM_LEN2_  0x02
#define _WIZCHIP_SPI_FDM_LEN4_  0x03
//
// If you want to use SPI FDM mode, Feel free contact to WIZnet. 
// http://forum.wiznet.io
//
DMA_InitTypeDef		DMA_RX_InitStructure, DMA_TX_InitStructure;
#if _WIZCHIP_ == 6100
////////////////////////////////////////////////////////////////////////////////////////
#define BUS_DMA
void Bus_Read_DMA(uint32_t addr,uint8_t* pBuf, uint32_t len);
void Bus_Write_DMA(uint32_t addr,uint8_t* pBuf, uint32_t len);
#define _W6100_SPI_OP_          _WIZCHIP_SPI_VDM_OP_

/***
*@brief: send data in BUS mode 
*@parame: addr: register address
*         data: value write to register
*@return: none
*****/
void IINCHIP_BusSendData(uint32_t addr,uint8_t data)
{
	 *((volatile uint8_t*)addr) = data;

}
/***
*@brief: read data in BUS mode 
*@parame: addr: register address
*         data: value read from register
*@return: register value
*****/
uint8_t IINCHIP_BusReadData(uint32_t addr)
{
	 return *((volatile uint8_t*)addr) ;

}

/***
*@brief:  pull down cs pin 
*@parame: none
*@return: none
*****/
void IINCHIP_CSoff(void)
{
  WIZ_CS(LOW);
}

/***
*@brief: pull up cs pin 
*@parame: none
*@return: none
*****/
void IINCHIP_CSon(void)
{
   WIZ_CS(HIGH);
}

/***
*@brief: send data in SPI mode 
*@parame: data: value write to register
*@return: none
*****/
uint8_t IINCHIP_SpiSendData(uint8_t dat)
{
   return(SPI2_SendByte(dat));
}


//////////////////////////////////////////////////
void WIZCHIP_WRITE(uint32_t AddrSel, uint8_t wb )
{
   uint8_t tAD[4];
   tAD[0] = (uint8_t)((AddrSel & 0x00FF0000) >> 16);
   tAD[1] = (uint8_t)((AddrSel & 0x0000FF00) >> 8);
   tAD[2] = (uint8_t)(AddrSel & 0x000000ff);
   tAD[3] = wb;

   //WIZCHIP_CRITICAL_ENTER();
  // WIZCHIP.CS._s_e_l_e_c_t_();
     IINCHIP_CSoff(); 
	
#if( (_WIZCHIP_IO_MODE_ == _WIZCHIP_IO_MODE_SPI_VDM_))
    tAD[2] |= (_W6100_SPI_WRITE_ | _W6100_SPI_OP_);
    //IINCHIP_SpiSendData( 0xf0);
	  IINCHIP_SpiSendData(tAD[0]);// Address byte 1
    IINCHIP_SpiSendData(tAD[1] );// Address byte 2
    IINCHIP_SpiSendData(tAD[2]);// Address byte 1
    IINCHIP_SpiSendData(tAD[3] );// Address byte 2
    

#elif ( (_WIZCHIP_IO_MODE_ == _WIZCHIP_IO_MODE_BUS_INDIR_) )
   IINCHIP_BusSendData(IDM_AR0 ,tAD[0] );
	 IINCHIP_BusSendData(IDM_AR1 ,tAD[1] );
	 IINCHIP_BusSendData(IDM_BSR ,tAD[2] );
	 IINCHIP_BusSendData(IDM_DR, tAD[3]);
#else
   #error "Unknown _WIZCHIP_IO_MODE_ in W5100. !!!"
#endif
   IINCHIP_CSon(); 
  // WIZCHIP.CS._d_e_s_e_l_e_c_t_();
  // WIZCHIP_CRITICAL_EXIT();
}

uint8_t  WIZCHIP_READ(uint32_t AddrSel)
{
   uint8_t ret;
   uint8_t tAD[3];
   tAD[0] = (uint8_t)((AddrSel & 0x00FF0000) >> 16);
   tAD[1] = (uint8_t)((AddrSel & 0x0000FF00) >> 8);
   tAD[2] = (uint8_t)(AddrSel & 0x000000ff);

  // WIZCHIP_CRITICAL_ENTER();
  // WIZCHIP.CS._s_e_l_e_c_t_();
 IINCHIP_CSoff(); 
#if( (_WIZCHIP_IO_MODE_ ==  _WIZCHIP_IO_MODE_SPI_VDM_))
   tAD[2] |= (_W6100_SPI_READ_ | _W6100_SPI_OP_);
   IINCHIP_SpiSendData( tAD[0]);	
   IINCHIP_SpiSendData( tAD[1]);
   IINCHIP_SpiSendData( tAD[2]);	// ƶ
   ret = IINCHIP_SpiSendData(0x00);      
#elif ( (_WIZCHIP_IO_MODE_ == _WIZCHIP_IO_MODE_BUS_INDIR_) )
   IINCHIP_BusSendData(IDM_AR0 ,tAD[0]);
	 IINCHIP_BusSendData(IDM_AR1 ,tAD[1]);
	 IINCHIP_BusSendData(IDM_BSR ,tAD[2] );
	 ret = IINCHIP_BusReadData(IDM_DR);
#else
   #error "Unknown _WIZCHIP_IO_MODE_ in W6100. !!!"   
#endif

  // WIZCHIP.CS._d_e_s_e_l_e_c_t_();
  // WIZCHIP_CRITICAL_EXIT();
	 IINCHIP_CSon(); 
   return ret;
}

void WIZCHIP_WRITE_BUF(uint32_t AddrSel, uint8_t* pBuf, datasize_t len)
{
	 uint16_t idx = 0;		// idxΪдĵڼ
   uint8_t tAD[3];
   tAD[0] = (uint8_t)((AddrSel & 0x00FF0000) >> 16);
   tAD[1] = (uint8_t)((AddrSel & 0x0000FF00) >> 8);
   tAD[2] = (uint8_t)(AddrSel & 0x000000ff);

   IINCHIP_CSoff();  
   // WIZCHIP_CRITICAL_ENTER();
   //WIZCHIP.CS._s_e_l_e_c_t_();

#if((_WIZCHIP_IO_MODE_ == _WIZCHIP_IO_MODE_SPI_VDM_))
   tAD[2] |= (_W6100_SPI_WRITE_ | _W6100_SPI_OP_);
   //	printf("write\r\n");																
   if(len == 0) printf("Unexpected2 length 0\r\n");			// дΪգlenʾдݵĳ
   IINCHIP_SpiSendData( tAD[0]);		// ַΣṩ16λƫƵַ0000 0000 0000 0000
   IINCHIP_SpiSendData( tAD[1]);		// ƶΣ8λ0000 0000 5λBSBλΪ00000ʾͨüĴ
   IINCHIP_SpiSendData( tAD[2]);    // ƶ+4(0000 0100 RWBλ1ʾд룬OMλΪ00ʾSPIģʽΪVDM)
   for(idx = 0; idx < len; idx++)                				// ݶΣдֵ
   {
     IINCHIP_SpiSendData( pBuf[idx]);											// MCUͨSPI
   }

#elif ( (_WIZCHIP_IO_MODE_ == _WIZCHIP_IO_MODE_BUS_INDIR_) )
	 #if !defined(BUS_DMA)
	 
   IINCHIP_BusSendData(IDM_AR0 ,tAD[0]);
	 IINCHIP_BusSendData(IDM_AR1 ,tAD[1]);
	 	 IINCHIP_BusSendData(IDM_BSR ,tAD[2] );
	 for(idx = 0; idx < len; idx++)                // Write data in loop
   {
		 IINCHIP_BusSendData(IDM_DR, pBuf[idx]);
	 }
	 #else
	  IINCHIP_BusSendData(IDM_AR0 ,tAD[0]);
	  IINCHIP_BusSendData(IDM_AR1 ,tAD[1]);
	 	IINCHIP_BusSendData(IDM_BSR ,tAD[2] );

      Bus_Write_DMA(IDM_DR,pBuf,len);	
	 #endif
#else
   #error "Unknown _WIZCHIP_IO_MODE_ in W6100. !!!!"
#endif

  // WIZCHIP.CS._d_e_s_e_l_e_c_t_();
  // WIZCHIP_CRITICAL_EXIT();
	IINCHIP_CSon(); 
}

void WIZCHIP_READ_BUF (uint32_t AddrSel, uint8_t* pBuf, datasize_t len)
{
   uint8_t tAD[3];
	 uint16_t idx = 0;		
   tAD[0] = (uint8_t)((AddrSel & 0x00FF0000) >> 16);
   tAD[1] = (uint8_t)((AddrSel & 0x0000FF00) >> 8);
   tAD[2] = (uint8_t)(AddrSel & 0x000000ff);

  // WIZCHIP_CRITICAL_ENTER();
  // WIZCHIP.CS._s_e_l_e_c_t_();
	IINCHIP_CSoff();        
if(len == 0)																				// lenΪȡݵĳ
  {	
    printf("Unexpected2 length 0\r\n");								// ȡݳΪ0
  }
#if((_WIZCHIP_IO_MODE_ == _WIZCHIP_IO_MODE_SPI_VDM_))
	
	
   tAD[2] |= (_W6100_SPI_READ_ | _W6100_SPI_OP_);
   IINCHIP_SpiSendData(tAD[0]);		// ַ
  IINCHIP_SpiSendData( tAD[1]);		// ƶ
  IINCHIP_SpiSendData( tAD[2]);    		// ƶ
  for(idx = 0; idx < len; idx++)                    	// ݶΣȡֵ
  {
    pBuf[idx] = IINCHIP_SpiSendData(0x00);							// MCUͨSPI͹ݴbuf
  }
#elif ( (_WIZCHIP_IO_MODE_ == _WIZCHIP_IO_MODE_BUS_INDIR_) )
	#if !defined(BUD_DMA)
     IINCHIP_BusSendData(IDM_AR0 ,tAD[0]);
	   IINCHIP_BusSendData(IDM_AR1 ,tAD[1]);
	 	 IINCHIP_BusSendData(IDM_BSR ,tAD[2] );
	   for(idx = 0; idx < len; idx++)                // Write data in loop
     {
		   pBuf[idx] =IINCHIP_BusReadData(IDM_DR);
	   }
	 #else
	   IINCHIP_BusSendData(IDM_AR0 ,tAD[0]);
	   IINCHIP_BusSendData(IDM_AR1 ,tAD[1]);
	 	 IINCHIP_BusSendData(IDM_BSR ,tAD[2] );
#if 0		
		 for(idx = 0; idx < len; idx++)
		 {
	    Bus_Read_DMA(IDM_DR,pBuf+idx,1);
		 }
#endif	
      Bus_Read_DMA(IDM_DR,pBuf,len);		 
   #endif	 
	
#else
   #error "Unknown _WIZCHIP_IO_MODE_ in W6100. !!!!"
#endif
   //WIZCHIP.CS._d_e_s_e_l_e_c_t_();
   //WIZCHIP_CRITICAL_EXIT();
	 IINCHIP_CSon();  
}

datasize_t getSn_TX_FSR(uint8_t sn)
{
   datasize_t prev_val=-1,val=0;
   do
   {
      prev_val = val;
      val = WIZCHIP_READ(W6100_Sn_TX_FSR_(sn));
      val = (val << 8) + WIZCHIP_READ(WIZCHIP_OFFSET_INC(W6100_Sn_TX_FSR_(sn),1));
   }while (val != prev_val);
   return val;
}

datasize_t getSn_RX_RSR(uint8_t sn)
{
   datasize_t prev_val=-1,val=0;
   do
   {
      prev_val = val;
      val = WIZCHIP_READ(W6100_Sn_RX_RSR_(sn));
      val = (val << 8) + WIZCHIP_READ(WIZCHIP_OFFSET_INC(W6100_Sn_RX_RSR_(sn),1));
   }while (val != prev_val);
   return val;
}

void wiz_send_data(uint8_t sn, uint8_t *wizdata, datasize_t len)
{
   datasize_t ptr = 0;
   uint32_t addrsel = 0;
   ptr = getSn_TX_WR(sn);
   addrsel = ((uint32_t)ptr << 8) + WIZCHIP_TXBUF_BLOCK(sn);
   WIZCHIP_WRITE_BUF(addrsel,wizdata, len);
   ptr += len;
   setSn_TX_WR(sn,ptr);
}

void wiz_recv_data(uint8_t sn, uint8_t *wizdata, datasize_t len)
{
   datasize_t ptr = 0;
   uint32_t addrsel = 0;
   if(len == 0) return;
   ptr = getSn_RX_RD(sn);
   addrsel = ((uint32_t)ptr << 8) + WIZCHIP_RXBUF_BLOCK(sn);
   WIZCHIP_READ_BUF(addrsel, wizdata, len);
   ptr += len;
   setSn_RX_RD(sn,ptr);
}

void wiz_recv_ignore(uint8_t sn, datasize_t len)
{
   setSn_RX_RD(sn,getSn_RX_RD(sn)+len);
}


/// @cond DOXY_APPLY_CODE
#if (_PHY_IO_MODE_ == _PHY_IO_MODE_MII_)
/// @endcond
void wiz_mdio_write(uint8_t phyregaddr, uint16_t var)
{
   setPHYRAR(phyregaddr);
   setPHYDIR(var);
   setPHYACR(PHYACR_WRITE);
   while(getPHYACR());  //wait for command complete
}

uint16_t wiz_mdio_read(uint8_t phyregaddr)
{
   setPHYRAR(phyregaddr);
   setPHYACR(PHYACR_READ);
   while(getPHYACR());  //wait for command complete
   return getPHYDOR();
}
/// @cond DOXY_APPLY_CODE
#endif
/// @endcond
void Bus_Write_DMA(uint32_t addr,uint8_t* pBuf, uint32_t len)
{
       RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);    

        /*??DMA???*/
               

        /*??:???SendBuff???ReceiveBuff*/        
        DMA_RX_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;    

        

        /*ReceiveBuff????*/      
        DMA_RX_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Enable; 

        /*SENDBUFF_SIZE????*/
        DMA_RX_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Disable;    

        /*ReceiveBuff????*/    
        DMA_RX_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;

        /*SENDBUFF_SIZE????*/
        DMA_RX_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;     

        /*DMA??:????*/
        DMA_RX_InitStructure.DMA_Mode = DMA_Mode_Normal ;

        /*???:?*/    
        DMA_RX_InitStructure.DMA_Priority = DMA_Priority_Medium;  

        /*??????????    */
        DMA_RX_InitStructure.DMA_M2M = DMA_M2M_Enable;
				DMA_RX_InitStructure.DMA_BufferSize = len;
		    DMA_RX_InitStructure.DMA_MemoryBaseAddr =(uint32_t)addr;
		    DMA_RX_InitStructure.DMA_PeripheralBaseAddr =(uint32_t)pBuf;
    
		    DMA_Init(DMA1_Channel5, &DMA_RX_InitStructure);

		    DMA_Cmd(DMA1_Channel5, ENABLE);
		    /* Waiting for the end of Data Transfer */
		    while(DMA_GetFlagStatus(DMA1_FLAG_TC5) == RESET);
		    DMA_ClearFlag(DMA1_FLAG_TC5);
		    DMA_Cmd(DMA1_Channel5, DISABLE);
}
void Bus_Read_DMA(uint32_t addr,uint8_t* pBuf, uint32_t len)
{
	        /*DMA*/
        RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);             

        /*??:???SendBuff???ReceiveBuff*/        
        DMA_RX_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;    

        

        /*ReceiveBuff????*/      
        DMA_RX_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Enable; 

        /*SENDBUFF_SIZE????*/
        DMA_RX_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Disable;    

        /*ReceiveBuff*/    
        DMA_RX_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;

        /*SENDBUFF_SIZE????*/
        DMA_RX_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;     

        /*DMA??:????*/
        DMA_RX_InitStructure.DMA_Mode = DMA_Mode_Normal ;

        /*???:?*/    
        DMA_RX_InitStructure.DMA_Priority = DMA_Priority_Medium;  

        /*??????????    */
        DMA_RX_InitStructure.DMA_M2M = DMA_M2M_Enable;
				DMA_RX_InitStructure.DMA_BufferSize = len;
		    DMA_RX_InitStructure.DMA_MemoryBaseAddr =(uint32_t)addr;
		    DMA_RX_InitStructure.DMA_PeripheralBaseAddr =(uint32_t)pBuf;
    
		    DMA_Init(DMA1_Channel4, &DMA_RX_InitStructure);

		    DMA_Cmd(DMA1_Channel4, ENABLE);
		    /* Waiting for the end of Data Transfer */
		    while(DMA_GetFlagStatus(DMA1_FLAG_TC4) == RESET);
		    DMA_ClearFlag(DMA1_FLAG_TC4);
		    DMA_Cmd(DMA1_Channel4, DISABLE);
}
////////////////////////////////////////////////////////////////////////////////////////
#endif
